

#ifndef _Waypoint_H__
#define _Waypoint_H__

#include "OrxonoxPrereqs.h"
#include "StaticEntity.h"
#include "overlays/OverlaysPrereqs.h"

#include <map>
#include <string>

#include "util/OgreForwardRefs.h"
#include "tools/interfaces/Tickable.h"
#include "interfaces/RadarListener.h"
#include "overlays/OrxonoxOverlay.h"

namespace orxonox
{
    /**
    @brief
        The StaticEntity is the simplest derivative of the @ref orxonox::WorldEntity class. This means all StaticEntity instances also have
        a position in space, a mass, a scale, a frication, ... because every StaticEntity is a WorldEntity. You can attach StaticEntities to eachother ike @ref orxonox::WorldEntity WorldEntities.

        In contrast to the MobileEntity the StaticEntity cannot move with respect to the parent to which it is attached. That's why
        it is called StaticEntity. It will keep the same position (always with respect to its parent) forever unless you call the
        function @see setPosition to changee it.

        A StaticEntity can only have the collisition type WorldEntity::None or WorldEntity::Static. The collsion types WorldEntity::Dynamic and WorldEntity::Kinematic are illegal.
    */

    class _OrxonoxExport Waypoint : public StaticEntity, public RadarListener
    {
        public:
            Waypoint(Context* context);
            virtual ~Waypoint();

            virtual void addObject(RadarViewable* object) override;
            virtual void removeObject(RadarViewable* viewable) override;
            virtual void objectChanged(RadarViewable* viewable) override;

            using StaticEntity::setPosition;
            using StaticEntity::setOrientation;

            virtual void setPosition(const Vector3& position) override;
            virtual void setOrientation(const Quaternion& orientation) override;

            virtual void positionChanged() override { }
            virtual void radarTick(float dt) override {}

            virtual inline float getRadarSensitivity() const override
                { return 1.0f; }

            inline unsigned int getMarkerLimit() const
                { return this->markerLimit_; }

            static void selectClosestTarget();
            static void selectNextTarget();



        private:
            void registerVariables();
            //virtual bool isCollisionTypeLegal(CollisionType type) const override;

            // network callbacks
            inline void positionChanged()
                { this->setPosition(this->getPosition()); }
            inline void orientationChanged()
                { this->setOrientation(this->getOrientation()); }

            // Bullet btMotionState related
            virtual void setWorldTransform(const btTransform& worldTrans) override;
            virtual void getWorldTransform(btTransform& worldTrans) const override;
    };
}

#endif /* _StaticEntity_H__ */
